#*************************************************************
# Progamm:			fluisa_wlgl_inst_1d
# Sinn des Programms: 		Löser der instationären Wärmeleitungsgleichung 1d
# Sinn der Einheit  :		Hauptprogramm	
# Name dieser Datei:		fluisa_wlgl_inst_1d_haupt.py
# Versionsnummer:		1
# Wird aufgerufen von:		-
# Ruft auf:			steuer.py, gitter.py, rand_bed.py, koeff_matrix.py, loeser_lgs_3diag.py, ausgabe.py
# Nummer im Programmpaket:	1
# von:				7			
# Autor: 			Armin Ulrich
# Erster Lauf: 			12.04.2018
# Letzte Änderung: 		18.01.2023
#*************************************************************

#*************************************************************
# Importieren der Klassen zur Zeitdokumentation
#*************************************************************

import time
import datetime

import numpy as np

#*************************************************************
# Importieren der Klassen
#*************************************************************
from steuer import Steuer
from gitter import Gitter
from rand_bed import Rand_bed
from koeff_matrix import Koeff_matrix
from loeser_lgs_3diag import Löser_lgs_3diag
from ausgabe import Ausgabe

#*************************************************************
# Belegung der Klassen
#*************************************************************
steuer = Steuer()
gitter = Gitter()
rand_bed = Rand_bed()
koeff_matrix = Koeff_matrix()
löser_lgs_3diag = Löser_lgs_3diag()
ausgabe = Ausgabe()

#*************************************************************
# Berechnung der Startzeit des Programmes mit:
# a) datetime
# b) time
# und Konvertieren der Startzeit zu einem String "datum_zeit"
#*************************************************************
dt_start = datetime.datetime.now()
lt_start = time.localtime()
jahr, monat, tag, stunde, minute, sekunde  = lt_start[0:6]
datum_zeit = "Datum: " +str(tag)+"."+str(monat)+"."+str(jahr)+"\n"+"Zeit: "+str(stunde)+":"+str(minute)+":"+str(sekunde)

#*************************************************************
# Deklarierung Prozeßausgabeliste
#*************************************************************
prozeß_ausgabe_liste = list(range(10))
for i in range(10):
  prozeß_ausgabe_liste[i] = "Nicht durchgeführt!"
  

#*************************************************************
# Ausgabe an
#*************************************************************
print("Wärmeleitungsgleichung,")
print("1d, instationär")

#*************************************************************
# öffnen der Log-Datei
#*************************************************************
fobj_log = open("fluisa_wlgl_inst_1d.log","w")

#*************************************************************
# Schreiben von Datum und Zeit des Programmstarts 
# in die Logdatei 
#*************************************************************
fobj_log.write("Anfang: "+ "\n" + datum_zeit)
fobj_log.write("\n")

#*************************************************************
# Prüfung, ob Steuerdatei erfolgreich geöffnet wurde
#*************************************************************
if(steuer.steuer_input()):
  fobj_log.write(steuer.rückgabeliste[1] + "\n")
  if(steuer.rückgabeliste[2] == True):
    if(steuer.rückgabeliste[3] == True):
      print(steuer.rückgabeliste[4])
      fobj_log.write(steuer.rückgabeliste[4] + "\n")
    else:
      print(steuer.rückgabeliste[4])  
      fobj_log.write(steuer.rückgabeliste[4] + "\n")

#*************************************************************
# Prüfung, ob das Gitter erfolgreich geladen wurde
#*************************************************************
  if(gitter.gitter_input(steuer.gitter_datei, steuer.gitter_ascii_bin)):
    print("Gitter wurde geladen")
    fobj_log.write("Gitter wurde geladen\n")

    T_feld = np.zeros((5, gitter.m))

#*************************************************************
# Einlesen der Randbedingungen
#*************************************************************
    if rand_bed.rand_bed_input(steuer.rand_datei):

#*************************************************************
# Belegen der Koeffizientenmatrix
#*************************************************************
      if koeff_matrix.matrix(steuer.lambada,steuer.c,steuer.rho,steuer.delta_t, steuer.theta,gitter.m, gitter.delta_x, gitter.A_gf, gitter.Delta_x, rand_bed.rand_art, rand_bed.rand_wert):
        if koeff_matrix.T_initial(steuer.T_start, gitter.m):
          

#*************************************************************
# Deklarieren und Initialisieren der Felder im Löser
# eine derartige Trennung verhindert bie instationären Problemen,
# daß in einer Zeitschleife die Felder wiederholt deklariert und initialisiert werden.
#*************************************************************
          if löser_lgs_3diag.feld_init(gitter.m):
            T_alt = koeff_matrix.T_alt
            t = steuer.t_start
            löser_lgs_3diag.rückgabeliste_iterationen_max = 0
#*************************************************************
# Zeitschleife
#*************************************************************
            while(t < steuer.t_ende):

              if koeff_matrix.matrix_instationär(steuer.delta_t,steuer.theta,T_alt, gitter.m,gitter.delta_x,gitter.A_gf,gitter.V):
               
#*************************************************************
# Lösen des GLS der Koeffizientenmatrix
#*************************************************************
#        T = löser_lgs_3diag.gs_3diag(gitter.m, steuer.max_iter, steuer.eps, steuer.omega, steuer.resi_max, koeff_matrix.aw, koeff_matrix.ap, koeff_matrix.ae, koeff_matrix.s, steuer.T_init)
                if löser_lgs_3diag.gs_3diag(gitter.m, steuer.max_iter, steuer.eps, steuer.omega, steuer.resi_max, koeff_matrix.aw, koeff_matrix.ap, koeff_matrix.ae, koeff_matrix.s,\
                  steuer.T_init):
                  T = löser_lgs_3diag.phi
                  prozeß_ausgabe_liste[8] = löser_lgs_3diag.rückgabeliste[1]
                  if löser_lgs_3diag.rückgabeliste[2] >= löser_lgs_3diag.rückgabeliste_iterationen_max:
                    löser_lgs_3diag.rückgabeliste_iterationen_max = löser_lgs_3diag.rückgabeliste[2]
                    prozeß_ausgabe_liste[9] = löser_lgs_3diag.rückgabeliste[2]
                  #print("iterationen:", prozeß_ausgabe_liste[7]) 
  
                  if(t>= steuer.t_ausgabe_1) & (t< steuer.t_ausgabe_1 + steuer.delta_t):
                    ausgabe.graphik_ausgabe_T(gitter.m,gitter.x, T, round(t,4),steuer.graphik_datei_ausgabe_wähler, steuer.zwischen_graphik_datei_name, steuer.graphik_datei_format,\
                    steuer.graphik_titel)
                    T_feld[0] = T #T_list[0] = T 

                  if(t>= steuer.t_ausgabe_2) & (t< steuer.t_ausgabe_2 + steuer.delta_t):
                    ausgabe.graphik_ausgabe_T(gitter.m,gitter.x, T, round(t,4),steuer.graphik_datei_ausgabe_wähler, steuer.zwischen_graphik_datei_name, steuer.graphik_datei_format,\
                    steuer.graphik_titel)
                    T_feld[1] = T#T_list[1] = T 

                  if(t>= steuer.t_ausgabe_3) & (t< steuer.t_ausgabe_3 + steuer.delta_t):
                    ausgabe.graphik_ausgabe_T(gitter.m, gitter.x, T, round(t,4),steuer.graphik_datei_ausgabe_wähler, steuer.zwischen_graphik_datei_name, steuer.graphik_datei_format,\
                    steuer.graphik_titel)
                    T_feld[2] = T#T_list[2] = T 

                  if(t>= steuer.t_ausgabe_4) & (t< steuer.t_ausgabe_4 + steuer.delta_t):
                    ausgabe.graphik_ausgabe_T(gitter.m, gitter.x, T, round(t,4),steuer.graphik_datei_ausgabe_wähler, steuer.zwischen_graphik_datei_name, steuer.graphik_datei_format,\
                    steuer.graphik_titel)
                    T_feld[3] = T#T_list[2] = T 
              
                T_alt = T
                t += steuer.delta_t
              else:
                print("Instationärer Teil der Koeffizientenmatrix wurde nicht erstellt.")
                fobj_log.write( "Instationärer Teil der Koeffizientenmatrix wurde nicht erstellt.\n")
              prozeß_ausgabe_liste[7] = koeff_matrix.rückgabeliste_matrix_inst[1]            

            T_feld[4] = T 

            if steuer.graphik_ausgabe_wähler: 
              ausgabe.graphik_ausgabe_T(gitter.m, gitter.x, T, round(t,4),steuer.graphik_datei_ausgabe_wähler, steuer.zwischen_graphik_datei_name, steuer.graphik_datei_format,\
              steuer.graphik_titel)


#*************************************************************
# Ausgabe der Lösung auf Schirm, in Datei, als Graphik 
#*************************************************************
            if steuer.schirm_ausgabe_wähler : 
              ausgabe.schirm_ausgabe(gitter.m, gitter.x, T_feld) 
            if steuer.datei_ausgabe_wähler: 
              ausgabe.datei_ausgabe(steuer.ausgabe_datei,gitter.m, gitter.x, T_feld)


            ausgabe.graphik_ausgabe_T_alle_zeiten(gitter.m, gitter.x,T_feld, steuer.t_ausgabe_1, steuer.t_ausgabe_2, steuer.t_ausgabe_3, steuer.t_ausgabe_4, steuer.t_ende, \
            steuer.graphik_datei_ausgabe_wähler,steuer.graphik_datei_name , steuer.graphik_datei_format )
    
#*************************************************************
# Fehler beim Einlesen der Gitterdatei
#************************************************************* 
          #print("Fehler beim Errechnen der Lösung")    
          #fobj_log.write("Fehler beim Errechnen der Lösung\n") 
            prozeß_ausgabe_liste[8] = löser_lgs_3diag.rückgabeliste[1]
            prozeß_ausgabe_liste[9] = löser_lgs_3diag.rückgabeliste_iterationen_max 
          else:
            print("Fehler beim Deklarieren des Phi-Feldes im Löser")    
            fobj_log.write("Fehler beim Deklarieren des Phi-Feldes im Löser\n")
          prozeß_ausgabe_liste[6] = löser_lgs_3diag.rückgabeliste_feld_init[1]
        else:
          print("Koeffizientenmatrix wurde nicht erstellt.")
          fobj_log.write( "Koeffizientenmatrix wurde nicht erstellt.\n")
        prozeß_ausgabe_liste[5] = koeff_matrix.rückgabeliste[1]           
      else:
        print("Fehler beim Initialisieren des T-Feldes in der Koeff-matrix")    
        fobj_log.write("Fehler beim Initialisieren des T-Feldes in der Koeff-matrix\n")  
      prozeß_ausgabe_liste[4] = koeff_matrix.rückgabeliste_T_feld_initialisierung[1]                  
    else:
      print("Randbedingungen wurden nicht geladen.")
      fobj_log.write( "Randbedingungen wurden nicht geladen.\n")    
    prozeß_ausgabe_liste[3] = rand_bed.rückgabeliste[1]      
#*************************************************************
# Fehler beim Einlesen der Gitterdatei
#************************************************************* 
  else:
    print("Gitter wurde nicht geladen")
    fobj_log.write( "Gitter wurde nicht geladen\n")
  prozeß_ausgabe_liste[2] = gitter.rückgabeliste[1]    
#*************************************************************
# Fehler beim Einlesen der Steuerdatei
#*************************************************************    
else:
  print(steuer.rückgabeliste[1])
  fobj_log.write( steuer.rückgabeliste[1]+"\n")    
prozeß_ausgabe_liste[0] = steuer.rückgabeliste[1]
prozeß_ausgabe_liste[1] = steuer.rückgabeliste[4] 
#*************************************************************
# Berechnung der Endzeit des Programmes mit:
# a) datetime
# b) time
# und Konvertieren der Endzeit zu einem String "datum_zeit"
#*************************************************************
lt_end = time.localtime()
jahr, monat, tag, stunde, minute, sekunde  = lt_end[0:6]
datum_zeit = "Datum: "+str(tag)+"."+str(monat)+"."+str(jahr)+"\n"+"Zeit: "+str(stunde)+":"+str(minute)+":"+str(sekunde)
dt_end = datetime.datetime.now()

#*************************************************************
# Berechnung der Rechendauer mit dem Zeitdifferenzoperator 
# des Moduls datetime
#*************************************************************
dauer = dt_end - dt_start

#*************************************************************
# Schreiben von Datum und Zeit des Programmendes und der 
# Rechendauer in die Logdatei 
#*************************************************************
fobj_log.write("Ende: "+ "\n" +datum_zeit + "\n")
fobj_log.write("Rechendauer: "+ "\n" + str(dauer))


#*************************************************************
# Schließen der Logdatei
#*************************************************************
fobj_log.close()
#*************************************************************
#Programmende
#*************************************************************
#*************************************************************
# Importieren der Klasse tkinter für GUI-Elemente
#*************************************************************
import tkinter
main = tkinter.Tk()

#*************************************************************
#Funktion zum Auflösen der Benutzeroberfläche
#*************************************************************
def ende():
  main.destroy()

#*************************************************************
# GUI-Fuktion
#*************************************************************    
             
lb_prozeß_ausgabe_steuer = tkinter.Label(main, text = prozeß_ausgabe_liste[0])
lb_prozeß_ausgabe_steuer.pack()
lb_prozeß_ausgabe_steuer_änderung = tkinter.Label(main, text = prozeß_ausgabe_liste[1])
lb_prozeß_ausgabe_steuer_änderung.pack()
lb_prozeß_ausgabe_gitter = tkinter.Label(main, text = prozeß_ausgabe_liste[2])
lb_prozeß_ausgabe_gitter.pack()
lb_prozeß_ausgabe_rand = tkinter.Label(main, text = prozeß_ausgabe_liste[3])
lb_prozeß_ausgabe_rand.pack()
lb_prozeß_ausgabe_koeff_matrix_T_init = tkinter.Label(main, text = prozeß_ausgabe_liste[4])
lb_prozeß_ausgabe_koeff_matrix_T_init.pack()
lb_prozeß_ausgabe_koeff_matrix = tkinter.Label(main, text = prozeß_ausgabe_liste[5])
lb_prozeß_ausgabe_koeff_matrix.pack()
lb_prozeß_ausgabe_löser_phi_init = tkinter.Label(main, text = prozeß_ausgabe_liste[6])
lb_prozeß_ausgabe_löser_phi_init.pack()
lb_prozeß_ausgabe_koeff_matrix_inst = tkinter.Label(main, text = prozeß_ausgabe_liste[7])
lb_prozeß_ausgabe_koeff_matrix_inst.pack()
lb_prozeß_ausgabe_löser_phi = tkinter.Label(main, text = prozeß_ausgabe_liste[8])
lb_prozeß_ausgabe_löser_phi.pack()
lb_prozeß_ausgabe_ausgabe = tkinter.Label(main, text ="mit maximal " + str(prozeß_ausgabe_liste[9]) + " Iterationen")
lb_prozeß_ausgabe_ausgabe.pack()
#*************************************************************
# Button zur Beendigung des Programms
#*************************************************************   
bende = tkinter.Button(main, text = "Ende",command = ende)
bende.pack()
#*************************************************************
# Endlosschleife
#*************************************************************
main.mainloop()

